/* Rate-1/N, Convolutional Encoder with memory M (N<=8,M<=31) */

/* Brian Chen (bchen@alum.mit.edu)
   Massachusetts Institute of Technology
   October 1999 */


void convEnc( char x, int state, char *y, int *newState, char crate, char
	      mem, int *g)
/* x        --> input bit
   state    --> mem LSBs represent the state of encoder.
   y        --> crate LSBs represent the coded bits c_{crate-1},...,c_1, c_0.
   newState --> mem LSBs represent the new state.
   crate    --> Rate of encoder is 1/crate.
   mem      --> memory of encoder
   g        --> array of generators
                NOTE: Shift register operates from LEFT to RIGHT in this
		      implementation, as is standard.
*/
{
  int i, j, b, c, mask=0;
  
  /* Augment state with input bit on left.  Make sure only mem LSBs of state
     are non-zero. */
  for (i=0; i<mem; i++)
    mask = (mask<<1) ^ 1;
  state = state & mask;
  state = state^(x<<mem);


  /* Calculate the crate coded bits. */
  *y = 0;
  for (i=0; i<crate; i++)
    {
      /* Multiply generator polynomial with state vector. */
      b = state & g[i];

      /* XOR bits of b to get coded bit c_i */
      c = 0;
      for (j=0; j<mem+1; j++)
	c = c ^ ((b>>j)&1);

      /* Store c_i in i-th LSB position of y */
      *y = *y ^ (c<<i);
    }


  /* Right-shift current augmented state by one bit to get new state. */
  *newState = (state>>1);
}
