/* Gaussian Channel */

#include <math.h>

double ran1( long* );
double gasdev( long* );
long idum = 148;


void gaussian_ch( double *x, double *y, double sigma, int num_sym )
/* Add zero-mean gaussian noise with standard deviation sigma to x to form y.
   The number of symbols in x is num_sym */
{
  int i;

  for (i=0; i<num_sym; i++)
    y[i] = gasdev(&idum)*sigma + x[i];
}


/* Returns a gaussian random variable with zero mean and unit variance. */
double gasdev( long *idum )
/* (C) Copr. 1986-92 Numerical Recipes Software @4A[Y"1i41kM. */
{
  static int iset=0;
  static double gset;
  double fac,rsq,v1,v2;

  if  (iset == 0)
    {
      do
	{
	  v1=2.0*ran1(idum)-1.0;
	  v2=2.0*ran1(idum)-1.0;
	  rsq=v1*v1+v2*v2;
	} while (rsq >= 1.0 || rsq == 0.0);
      fac=sqrt(-2.0*log(rsq)/rsq);
      gset=v1*fac;
      iset=1;
      return v2*fac;
    }
  else
    {
      iset=0;
      return gset;
    }
}

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double ran1(long *idum)
{
  int j;
  long k;
  static long iy=0;
  static long iv[NTAB];
  double temp;
  
  if (*idum <= 0 || !iy)
    {
      if (-(*idum) < 1)
	*idum=1;
      else
	*idum = -(*idum);
      for (j=NTAB+7;j>=0;j--)
	{
	  k=(*idum)/IQ;
	  *idum=IA*(*idum-k*IQ)-IR*k;
	  if (*idum < 0)
	    *idum += IM;
	  if (j < NTAB)
	    iv[j] = *idum;
	}
      iy=iv[0];
    }
  k=(*idum)/IQ;
  *idum=IA*(*idum-k*IQ)-IR*k;
  if (*idum < 0)
    *idum += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = *idum;
  if ((temp=AM*iy) > RNMX)
    return RNMX;
  else
    return temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX
/* (C) Copr. 1986-92 Numerical Recipes Software @4A[Y"1i41kM. */
